#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "usart.h"
#include "lcd.h"

void waitForOK(void);
void waitForHTTPResponse(void);
void getRate(char * currency);
char * getNonNumeric(char * str, uint8_t max);

//zapytanie do serwera
const char request[] = "GET /api/exchangerates/rates/a/%s/?format=json HTTP/1.1\r\n"
"Host: api.nbp.pl\r\n"
"Connection: close\r\n"
"\r\n";

char buffer[512];
uint16_t bufferIndex = 0;
volatile uint8_t ok = 0;
volatile uint8_t gotIP = 0;
volatile uint8_t response = 0;
volatile uint8_t httpResponseReady = 0;
FILE usartStream;

int main(void){
	DDRD |= _BV(DDD2);
	//zresetuj modu
	PORTD &= ~_BV(PD2);
	_delay_ms(10);
	PORTD |= _BV(PD2);
	lcdInit();
	lcdInitPrintf();
	usartInit(115200);
	sei();
	UCSRB |= _BV(RXCIE);
	fdev_setup_stream(&usartStream, &usartPut, &usartGet, _FDEV_SETUP_RW);
	printf("EUR: ");
	//poczekaj na poczenie z WiFi
	while(!gotIP);
	//wcz tryb jednopoczeniowy
	fprintf(&usartStream, "AT+CIPMUX=0\r\n");
	waitForOK();
	getRate("eur");
	lcdGotoXY(0, 1);
	printf("USD: ");
	getRate("usd");
    while(1);
}

void getRate(char * currency) {
	//pocz z serwerem
	fprintf(&usartStream, "AT+CIPSTART=\"TCP\",\"api.nbp.pl\",80\r\n");
	waitForOK();
	//przygotuj zapytanie
	fprintf(&usartStream, "AT+CIPSEND=%d\r\n", strlen(request) -2 + strlen(currency));
	waitForOK();
	//wylij zapytanie
	fprintf(&usartStream, request, currency);
	waitForHTTPResponse();
	//odszukaj warto "mid"
	char * mid = strstr(buffer, "mid");
	char * end = getNonNumeric(mid + 5, 10);
	*end = 0;
	printf(mid + 5);
	//rozcz si
	fprintf(&usartStream, "AT+CIPCLOSE\r\n");
	while(!response);
	response = 0;
}

char * getNonNumeric(char * str, uint8_t max) {
	for (uint8_t i = 0; i < max; i++) {
		char c = *str;
		if ((c < '0' || c > '9') && c != '.') return str;
		str++;
	}
	return 0;
}

void waitForOK(void) {
	while(!ok);
	ok = 0;
}

void waitForHTTPResponse(void) {
	while(!httpResponseReady);
	httpResponseReady = 0;
}

ISR(USART_RXC_vect) {
	static uint16_t size = 0;
	static uint8_t ipd = 0;
	uint8_t value = UDR;
	if (bufferIndex < sizeof(buffer)) {
		buffer[bufferIndex++] = value;
	}
	if (ipd) {
		if (size == 0) {
			if (value == ':') {
				//sprawd ile bajtw przyszo z serwera
				size = atoi(buffer + 5);
				bufferIndex = 0;
			}
		} else {
			if (bufferIndex == size) {
				//odebrano ca odpowied
				httpResponseReady = 1;
				size = 0;
				ipd = 0;
				bufferIndex = 0;
			}
		}
	} else {
		if (value == '\n') {
			buffer[bufferIndex] = 0;
			bufferIndex = 0;
			if (strncmp(buffer, "OK", 2) == 0) ok = 1;
			if (strncmp(buffer, "WIFI GOT IP", 11) == 0) gotIP = 1;
			response = 1;
		}
		if (bufferIndex == 5) {
			if (strncmp(buffer, "+IPD,", 5) == 0) ipd = 1;
		}
	}
}
